@@ -12,14 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import print_function
-
 import contextlib
 import datetime
 import errno
 import json
 import logging
 import os
+import queue
 import re
 import requests
 import shutil
@@ -29,14 +28,12 @@ import tempfile
 import threading
 import time
 
-from distutils.version import LooseVersion
 from distutils.version import StrictVersion
 import docker
 import git
 import jinja2
 from oslo_config import cfg
 from requests import exceptions as requests_exc
-import six
 
 
 # NOTE(SamYaple): Update the search path to prefer PROJECT_ROOT as the source
@@ -83,46 +80,39 @@ STATUS_ERRORS = (STATUS_CONNECTION_ERROR, STATUS_PUSH_ERROR,
 # is irrelevant. Otherwise all must match for skip to happen.
 UNBUILDABLE_IMAGES = {
     'aarch64': {
-        "cyborg-base",       # no binary package
         "monasca-grafana",   # no phantomJS on aarch64
         "opendaylight",      # no binary package
         "prometheus-mtail",  # no aarch64 binary
-        "telegraf",          # no binary package
+        "skydive-base",      # no aarch64 binary
     },
 
     'ppc64le': {
-        "cyborg-base",       # no binary package
         "elasticsearch",     # no binary package
         "grafana",           # no binary package
-        "kube-proxy",        # no binary
-        "helm-repository",   # no binary
         "monasca-grafana",   # no phantomJS
         "opendaylight",      # no binary package
         "prometheus-base",   # no ppc64le binaries
+        "skydive-base",      # no ppc64le binaries
         "telegraf",          # no binary package
         "xtrabackup",        # no binary package
     },
 
     'source': {
-        "helm-repository",   # chart repository is gone
-        "kuryr-libnetwork",  # Could not find a version that satisfies the
-                             # requirement jsonschema>=2.6.0,<3.0.0
         "tripleoclient",
     },
+
     'binary': {
-        "almanach-base",
         "bifrost-base",
         "blazar-base",
         "cyborg-base",
-        "dragonflow-base",
         "freezer-base",
-        "helm-repository",   # chart repository is gone
         "karbor-base",
         "kuryr-base",
         "masakari-base",
         "monasca-base",
         "monasca-thresh",
         "nova-mksproxy",
+        "openvswitch-netcontrold",
         "qinling-base",
         "searchlight-base",
         "solum-base",
@@ -143,87 +133,49 @@ UNBUILDABLE_IMAGES = {
     },
 
     'centos': {
-        "mongodb",
-        "ovsdpdk",
-    },
-
-    # NOTE(mgoddard): Mark images with missing dependencies as unbuildable for
-    # CentOS 8.
-    'centos8': {
-        "almanach-base",         # Dropped in master
-        "ceph-base",             # Missing Ceph repo
-        "dind",                  # Dropped in master
-        "dragonflow-base",       # Dropped in master
-        "elasticsearch",         # Missing elasticsearch repo
         "hacluster-pcs",         # Missing crmsh package
-        "helm-repository",       # Dropped in master
-        "kibana",                # Missing elasticsearch repo
-        "kube-base",             # Dropped in master
-        "kubernetes-entrypoint",  # Dropped in master
-        "kubetoolbox",           # Dropped in master
         "mongodb",               # Missing mongodb and mongodb-server packages
         "nova-spicehtml5proxy",  # Missing spicehtml5 package
         "opendaylight",          # Missing opendaylight repo
         "ovsdpdk",               # Not supported on CentOS
-        "rabbitmq-3.7.24",       # Required only for CentOS 7 to 8 migration
-        "sensu-base",            # Missing sensu package
+        "sensu-base",            # Dropped by CentOS
         "tgtd",                  # Not supported on CentOS 8
     },
 
-    'centos8+source': {
-        "cyborg-agent",          # opae-sdk does not support CentOS 8
-    },
-
     'debian': {
         "bifrost-base",  # tries to install 'mysql-server' which is not in
                          # Debian 'buster'
-        "cyborg-base",
-        "elasticsearch6",   # Only required for CentOS 8 migration.
-        "logstash6",        # Only required for CentOS 8 migration.
-        "kibana6",          # Only required for CentOS 8 migration.
         "monasca-grafana",  # FIXME(hrw): some ssl issues to fix
         "mongodb",
         "opendaylight",  # no binary package
         "ovsdpdk",
         "qdrouterd",
-        "rabbitmq-3.7.24",  # Required only for CentOS 7 to 8 migration
         "sensu-base",
-        "telegraf",      # no binary package
     },
+
     'ubuntu': {
-        "cyborg-base",
-        "elasticsearch6",   # Only required for CentOS 8 migration.
-        "logstash6",        # Only required for CentOS 8 migration.
-        "kibana6",          # Only required for CentOS 8 migration.
         "qdrouterd",  # There is no qdrouterd package for ubuntu bionic
-        "rabbitmq-3.7.24",  # Required only for CentOS 7 to 8 migration
-        "sensu-base",       # upstream dropped Ubuntu repositories
     },
 
-    'debian+aarch64': {
-        "skydive-base",  # no binary package
-    },
     'ubuntu+aarch64': {
-        "skydive-base",  # no binary package
+        "kibana",        # no binary package
+        "sensu-base",    # no binary package
     },
 
-    'debian+ppc64le': {
-        "skydive-base",  # no binary package
-    },
     'ubuntu+ppc64le': {
-        "skydive-base",  # no binary package
+        "sensu-base",    # no binary package
     },
 
     'centos+aarch64': {
         "elasticsearch",  # no binary package
         "fluentd",        # no binary package
-        "gnocchi-base",   # no python3-boto3 package in HA
         "hacluster-pcs",  # no binary package
         "influxdb",       # no binary package
         "kibana",         # no binary package
-        "kolla-toolbox",  # no erlang package
-        "rabbitmq",       # no erlang package
+        "kolla-toolbox",  # no Erlang package
         "mongodb",        # no binary package
+        "rabbitmq",       # no Erlang package
+        "telegraf",       # no binary package
     },
 
     'centos+ppc64le': {
@@ -241,9 +193,12 @@ UNBUILDABLE_IMAGES = {
         "neutron-server-opendaylight",
         "nova-serialproxy",
         "novajoin-base",
+        "senlin-conductor",  # no binary package
+        "senlin-health-manager",  # no binary package
         "tacker-base",
         "tripleoclient",
         "trove-base",
+        "neutron-mlnx-agent",
     },
 
     'ubuntu+binary': {
@@ -254,24 +209,13 @@ UNBUILDABLE_IMAGES = {
         "ironic-neutron-agent",
         "novajoin-base",
         "rally",
+        "senlin-conductor",  # no binary package
+        "senlin-health-manager",  # no binary package
         "tacker-base",
         "tripleoclient",
         "vitrage-base",
         "zaqar",
-    },
-
-    'debian+source': {
-        "almanach-base",     # Could not find a version that satisfies the
-                             # requirement Flask!=0.11,<1.0,>=0.10
-        "dragonflow-base",   # Could not find a version that satisfies the
-                             # requirement SQLAlchemy<1.1.0,>=1.0.10
-    },
-
-    'ubuntu+source': {
-        "almanach-base",     # Could not find a version that satisfies the
-                             # requirement Flask!=0.11,<1.0,>=0.10
-        "dragonflow-base",   # Could not find a version that satisfies the
-                             # requirement SQLAlchemy<1.1.0,>=1.0.10
+        "neutron-mlnx-agent",
     },
 }
 
@@ -371,8 +315,7 @@ class PushIntoQueueTask(task.Task):
 
     @property
     def name(self):
-        return 'PushIntoQueueTask(%s=>%s)' % (self.push_task.name,
-                                              self.push_queue)
+        return 'PushIntoQueueTask(%s)' % (self.push_task.name)
 
     def run(self):
         self.push_queue.put(self.push_task)
@@ -448,7 +391,6 @@ class BuildTask(DockerTask):
         self.conf = conf
         self.image = image
         self.push_queue = push_queue
-        self.nocache = not conf.cache
         self.forcerm = not conf.keep
         self.logger = image.logger
 
@@ -722,7 +664,7 @@ class WorkerThread(threading.Thread):
                 self.queue.put(task)
                 break
             try:
-                for attempt in six.moves.range(self.conf.retries + 1):
+                for attempt in range(self.conf.retries + 1):
                     if self.should_stop:
                         break
                     LOG.info("Attempt number: %s to run task: %s ",
@@ -770,6 +712,7 @@ class KollaWorker(object):
             self.debian_arch = 'ppc64el'
         self.images = list()
         self.openstack_release = conf.openstack_release
+        self.docker_healthchecks = conf.docker_healthchecks
         rpm_setup_config = ([repo_file for repo_file in
                              conf.rpm_setup_config if repo_file is not None])
         self.rpm_setup = self.build_rpm_setup(rpm_setup_config)
@@ -779,9 +722,7 @@ class KollaWorker(object):
         deb_base = ['ubuntu', 'debian']
         deb_type = ['source', 'binary']
 
-        if self.base in rh_base and self.base_tag.startswith('7'):
-            self.conf.distro_python_version = "2.7"
-        elif self.base in rh_base and self.base_tag.startswith('8'):
+        if self.base in rh_base:
             self.conf.distro_python_version = "3.6"
         elif self.base in ['debian']:
             self.conf.distro_python_version = "3.7"
@@ -789,15 +730,12 @@ class KollaWorker(object):
             self.conf.distro_python_version = "3.6"
         else:
             # Assume worst
-            self.conf.distro_python_version = "2.7"
+            self.conf.distro_python_version = "3.6"
 
         if self.conf.distro_package_manager is not None:
             package_manager = self.conf.distro_package_manager
         elif self.base in rh_base:
-            if LooseVersion(self.base_tag) >= LooseVersion('8'):
-                package_manager = 'dnf'
-            else:
-                package_manager = 'yum'
+            package_manager = 'dnf'
         elif self.base in deb_base:
             package_manager = 'apt'
         self.distro_package_manager = package_manager
@@ -833,6 +771,12 @@ class KollaWorker(object):
                 'Unknown install type'
             )
 
+        if (self.install_type == 'binary' and self.base == 'debian' and
+                self.base_arch != 'x86_64'):
+            LOG.info("Debian/binary target is available only for x86-64 "
+                     "due to lack of packages for other architectures.")
+            sys.exit(1)
+
         self.image_prefix = self.base + '-' + self.install_type + '-'
 
         self.regex = conf.regex
@@ -862,7 +806,6 @@ class KollaWorker(object):
             PROJECT_ROOT,
             os.path.join(sys.prefix, 'share/kolla'),
             os.path.join(sys.prefix, 'local/share/kolla'),
-            os.path.join(os.getenv('HOME', ''), '.local/share/kolla'),
             # NOTE(zioproto): When Kolla is used within a snap, the env var
             #                 $SNAP is the directory where the snap is mounted.
             #                 https://github.com/zioproto/snap-kolla
@@ -892,8 +835,8 @@ class KollaWorker(object):
 
         for config in rpm_setup_config:
             if config.endswith('.rpm'):
-                # RPM files can be installed with yum from file path or url
-                cmd = "RUN yum -y install {}".format(config)
+                # RPM files can be installed with dnf from file path or url
+                cmd = "RUN dnf -y install {}".format(config)
             elif config.endswith('.repo'):
                 if config.startswith('http'):
                     # Curl http://url/etc.repo to /etc/yum.repos.d/etc.repo
@@ -979,10 +922,11 @@ class KollaWorker(object):
 
         return {
             'debian_package_install': jinja_methods.debian_package_install,
+            'handle_repos': jinja_methods.handle_repos,
         }
 
     def get_users(self):
-        all_sections = (set(six.iterkeys(self.conf._groups)) |
+        all_sections = (set(self.conf._groups.keys()) |
                         set(self.conf.list_all_sections()))
         ret = dict()
         for section in all_sections:
@@ -1012,6 +956,7 @@ class KollaWorker(object):
                       'use_dumb_init': self.use_dumb_init,
                       'base_package_type': self.base_package_type,
                       'debian_arch': self.debian_arch,
+                      'docker_healthchecks': self.docker_healthchecks,
                       'supported_distro_release': supported_distro_release,
                       'install_metatype': self.install_metatype,
                       'image_prefix': self.image_prefix,
@@ -1103,22 +1048,18 @@ class KollaWorker(object):
                     filter_ += self.conf.profiles[profile]
 
         # mark unbuildable images and their children
-
-        # NOTE(mgoddard): Use a base of centos8 to allow a different set of
-        # unbuildable images for CentOS 8.
-        # TODO(mgoddard): Remove this after CentOS 8 transition.
         base = self.base
-        if base == 'centos' and self.distro_package_manager == 'dnf':
-            base = 'centos8'
 
         tag_element = r'(%s|%s|%s)' % (base,
                                        self.install_type,
                                        self.base_arch)
         tag_re = re.compile(r'^%s(\+%s)*$' % (tag_element, tag_element))
         unbuildable_images = set()
-        for set_tag in UNBUILDABLE_IMAGES:
-            if tag_re.match(set_tag):
-                unbuildable_images.update(UNBUILDABLE_IMAGES[set_tag])
+
+        if not self.conf.enable_unbuildable:
+            for set_tag in UNBUILDABLE_IMAGES:
+                if tag_re.match(set_tag):
+                    unbuildable_images.update(UNBUILDABLE_IMAGES[set_tag])
 
         if unbuildable_images:
             for image in self.images:
@@ -1306,7 +1247,7 @@ class KollaWorker(object):
                     installation['reference'] = self.conf[section]['reference']
             return installation
 
-        all_sections = (set(six.iterkeys(self.conf._groups)) |
+        all_sections = (set(self.conf._groups.keys()) |
                         set(self.conf.list_all_sections()))
 
         for path in self.docker_build_paths:
@@ -1403,7 +1344,7 @@ class KollaWorker(object):
             return
 
         def list_children(images, ancestry):
-            children = six.next(iter(ancestry.values()))
+            children = next(iter(ancestry.values()))
             for image in images:
                 if image.status not in [STATUS_MATCHED]:
                     continue
@@ -1437,11 +1378,7 @@ class KollaWorker(object):
         Return a list of Queues that have been organized into a hierarchy
         based on dependencies
         """
-        self.build_image_list()
-        self.find_parents()
-        self.filter_images()
-
-        queue = six.moves.queue.Queue()
+        build_queue = queue.Queue()
 
         for image in self.images:
             if image.status in (STATUS_UNMATCHED, STATUS_SKIPPED,
@@ -1453,10 +1390,10 @@ class KollaWorker(object):
             # Build all root nodes, where a root is defined as having no parent
             # or having a parent that is explicitly being skipped.
             if image.parent is None or image.parent.status == STATUS_SKIPPED:
-                queue.put(BuildTask(self.conf, image, push_queue))
+                build_queue.put(BuildTask(self.conf, image, push_queue))
                 LOG.info('Added image %s to queue', image.name)
 
-        return queue
+        return build_queue
 
 
 def run_build():
@@ -1480,8 +1417,17 @@ def run_build():
     kolla.setup_working_dir()
     kolla.find_dockerfiles()
     kolla.create_dockerfiles()
+    kolla.build_image_list()
+    kolla.find_parents()
+    kolla.filter_images()
 
     if conf.template_only:
+        for image in kolla.images:
+            if image.status == STATUS_MATCHED:
+                continue
+
+            shutil.rmtree(image.path)
+
         LOG.info('Dockerfiles are generated in %s', kolla.working_dir)
         return
 
@@ -1490,56 +1436,47 @@ def run_build():
     kolla.set_time()
 
     if conf.save_dependency:
-        kolla.build_image_list()
-        kolla.find_parents()
-        kolla.filter_images()
         kolla.save_dependency(conf.save_dependency)
         LOG.info('Docker images dependency are saved in %s',
                  conf.save_dependency)
         return
     if conf.list_images:
-        kolla.build_image_list()
-        kolla.find_parents()
-        kolla.filter_images()
         kolla.list_images()
         return
     if conf.list_dependencies:
-        kolla.build_image_list()
-        kolla.find_parents()
-        kolla.filter_images()
         kolla.list_dependencies()
         return
 
-    push_queue = six.moves.queue.Queue()
-    queue = kolla.build_queue(push_queue)
+    push_queue = queue.Queue()
+    build_queue = kolla.build_queue(push_queue)
     workers = []
 
     with join_many(workers):
         try:
-            for x in six.moves.range(conf.threads):
-                worker = WorkerThread(conf, queue)
+            for x in range(conf.threads):
+                worker = WorkerThread(conf, build_queue)
                 worker.setDaemon(True)
                 worker.start()
                 workers.append(worker)
 
-            for x in six.moves.range(conf.push_threads):
+            for x in range(conf.push_threads):
                 worker = WorkerThread(conf, push_queue)
                 worker.setDaemon(True)
                 worker.start()
                 workers.append(worker)
 
-            # sleep until queue is empty
-            while queue.unfinished_tasks or push_queue.unfinished_tasks:
+            # sleep until build_queue is empty
+            while build_queue.unfinished_tasks or push_queue.unfinished_tasks:
                 time.sleep(3)
 
             # ensure all threads exited happily
             push_queue.put(WorkerThread.tombstone)
-            queue.put(WorkerThread.tombstone)
+            build_queue.put(WorkerThread.tombstone)
         except KeyboardInterrupt:
             for w in workers:
                 w.should_stop = True
             push_queue.put(WorkerThread.tombstone)
-            queue.put(WorkerThread.tombstone)
+            build_queue.put(WorkerThread.tombstone)
             raise
 
     results = kolla.summary()
